import React, { Component } from 'react';
import { connect as dvaConnect } from 'dva';
import { getApp } from '../../data/app';

const connect = (modelCreator, { app, mapStateToProps, mapDispatchToProps, mergeProps, options }) => (Comp) => {
  const { name, model } = modelCreator();
  const { dispatchVar, namespaceVar } = (options || {});
  class StatefulComponent extends Component {

    componentWillMount() {
      (app || this.props.app || getApp()).model(model);
    }
    componentWillUnmount() {
      (app || this.props.app || getApp()).unmodel(model);
    }

    render() {
      const { children, ...rest } = this.props;
      const mapState = (state) => {
        const props = mapStateToProps ? mapStateToProps(state) : {};
        props[name] = state[model.namespace];
        props[namespaceVar || 'namespace'] = model.namespace;
        return props;
      };
      const mapDispatch = (dispatch) => {
        const extras = mapDispatchToProps ? mapDispatchToProps(dispatch) : {};
        return {
          ...extras,
          [dispatchVar || 'dispatchLocal'](action) {
            const { type, payload } = action;
            return dispatch({ type: `${model.namespace}/${type}`, payload });
          },
        };
      };
      const Output = dvaConnect(mapState, mapDispatch, mergeProps, options)(Comp);
      return (
        <Output {...rest}>
          { children }
        </Output>
      );
    }

  }
  return StatefulComponent;
};

export default connect;
